# -*- coding: utf-8 -*-
"""Definition of the DashboardManager content type
"""
from AccessControl import ClassSecurityInfo
from Products.Archetypes import atapi
from collective.portletpage.content import PortletPage, PortletPageSchema
from vs.dashboardmanager import config
from vs.dashboardmanager.interfaces import IDashboardManager
from zope.interface import implements

class DashboardManager(PortletPage):
    """a dashboardmanager"""
    
    implements(IDashboardManager)
    security = ClassSecurityInfo()

    meta_type = "DashboardManager"
    schema = PortletPageSchema

    title = atapi.ATFieldProperty('title')
    description = atapi.ATFieldProperty('description')
    
atapi.registerType(DashboardManager, config.PROJECTNAME)



################################################################
# vs.dashboardmanager
# (C) 2010, Veit Schiele & Andreas Jung
# Published under the GNU Public Licence V 2 (GPL 2)
################################################################

import operator 
from zope.component import getUtility, getMultiAdapter
try:
    from zope.app.container.interfaces import INameChooser # Plone < 4.3
except ImportError:
    from zope.container.interfaces import INameChooser # Plone >= 4.3
from Products.CMFCore.utils import getToolByName
from Products.CMFCore.interfaces import ISiteRoot
from Products.Five.browser import BrowserView
from Products.Five.browser.pagetemplatefile import ViewPageTemplateFile

from plone.portlets.interfaces import IPortletManager, IPortletAssignment, IPortletAssignmentMapping
from plone.app.portlets.utils import assignment_mapping_from_key

from vs.dashboardmanager import MF as _
from vs.dashboardmanager.logger import LOG

class Assignments(BrowserView):

    template = ViewPageTemplateFile('assignments.pt')

    def groups_for_js(self):
        """ Groups for Javascript """
        groups = self.getGroups()
        return str([group['id'] for group in groups])

    def getGroups(self): 
        """ Return PAS groups """
        search_view = self.context.restrictedTraverse('@@pas_search')
        if 'name' in self.request:
            # search by 'id' also seams to work through LDAP groups
            # instead of search by 'name'
            result = search_view.searchGroups(id=self.request.name)
        else:
            result = search_view.searchGroups()
        result = [r for r in result if not r['pluginid'] in ('auto_group',)]
        result = sorted(result, key=operator.itemgetter('title')) 
        return result

    def _get_memberids_from_request(self):
        """ Return a list of userids based on the 'group' and 'userid'
            parameters from the request.
        """

        userid = self.request.get('userid', '')
        group = self.request.get('group', '')

        gt = getToolByName(self.context, 'portal_groups')
        mt = getToolByName(self.context, 'portal_membership')

        if userid and group:
            return ()

        if not userid and not group:
            return ()

        site = getUtility(ISiteRoot)
        if userid:
            userids = [userid]
        else:
            acl = self.context.acl_users
            grp = acl.getGroupById(group)
            userids = grp.getMemberIds()

        return userids


    def push_assignment(self):
        """ Push all assignment portlet page assignment into the
            dashboard of a user or a group of users.
        """

        userids = self._get_memberids_from_request()
        if not userids:
            self.context.plone_utils.addPortalMessage(u'No members found', 'error')
            url = '%s/@@assignments?userid=%s&group=%s' % (self.context.absolute_url(),
                    self.request.get('userid', ''), self.request.get('group', ''))
            return self.request.response.redirect(url)

        dashboards_updated = dict()
        site = getUtility(ISiteRoot)
        for userid in userids:

            # iterate over all configured portlet managers (1-4) for PortletPage
            for i in range(1, 5):

                manager_name = 'vs.dashboardmanager.column%d' % i
                manager = getUtility(IPortletManager, name=manager_name, context= self.context)
                mapping = getMultiAdapter((self.context, manager), IPortletAssignmentMapping)

                # get hold of the user dashboard manager
                manager2 = getUtility(IPortletManager, name='plone.dashboard%d' % i)
                mapping2 = assignment_mapping_from_key(site, 
                                                      'plone.dashboard%d' % i, 
                                                       category='user', 
                                                       key=userid, 
                                                       create=True)

                ids2 = [tp[0] for tp in mapping2.items()]

                # and copy over the assignments
                for id, assignment in mapping.items():
                    assignment_copy=assignment._getCopy(assignment)
                    if not id in ids2:
                        mapping2[id] = assignment_copy
                        LOG.info('assigned portlet %s to %s' % (id, userid))
                        dashboards_updated[userid] = True

        if dashboards_updated:
            self.context.plone_utils.addPortalMessage(u'Dashboard(s) updated for members: %s' 
                                                      % ', '.join(dashboards_updated.keys()))
        else:
            self.context.plone_utils.addPortalMessage(u'No dashboards updated')
        return self.request.response.redirect(self.context.absolute_url())


    def remove_assignments(self):
        """ Remove a list of portlets from user dashboards """

        site = getUtility(ISiteRoot)
        for id in self.request.form.get('ids', []):
            # id = 'user=XXXX::manager=YYYY:portlet=ZZZZ'
            items = id.split('::')
            userid = items[0].split('=')[-1]
            portlet_manager_name = items[1].split('=')[-1]
            portlet_id = items[2].split('=')[-1]

            # get hold of the user dashboard manager
            portlet_manager = getUtility(IPortletManager, name=portlet_manager_name)
            mapping = assignment_mapping_from_key(site, 
                                                  portlet_manager_name,
                                                  category='user', 
                                                  key=userid, 
                                                  create=False)
            del mapping[portlet_id]
            mapping._p_changed = True

        self.context.plone_utils.addPortalMessage((u'label_portlets_removed_from_dashboard',
                                                    u'Portlets removed from user dashboard(s)'))

        url = '%s/@@view-assignments?userid=%s&group=%s' % (self.context.absolute_url(),
                self.request.get('userid', ''), self.request.get('group', ''))
        return self.request.response.redirect(url)

    def __call__(self, *args, **kw):
        return self.template(*args, **kw)


class ViewAssignments(Assignments):

    template = ViewPageTemplateFile('view-assignments.pt')

    def get_assignments(self):
        """ Return dashboard settings for a given group or a user """

        userids = self._get_memberids_from_request()
        site = getUtility(ISiteRoot)

        user2dash = list()
        for userid in userids:
            # iterate over the four portlet managers in each user dashboard
            dash2portlets = list()
            found = False
            for i in range(1, 5):
                # get hold of the user dashboard manager
                manager_name = 'plone.dashboard%d' % i
                manager = getUtility(IPortletManager, name=manager_name)
                mapping = assignment_mapping_from_key(site, 
                                                      manager_name,
                                                      category='user', 
                                                      key=userid, 
                                                      create=True)
                portlets = list()
                for portlet_id in mapping:
                    assignment = mapping[portlet_id]
                    parameters = dict([(k,v) for k,v in assignment.__dict__.items() if not k.startswith('_')])
                    portlets.append(dict(portlet_id=portlet_id, 
                                         assignment=assignment,
                                         parameters=parameters))
                    found = True

                dash2portlets.append(dict(manager=manager_name, portlets=portlets))
            user2dash.append(dict(userid=userid, dashboards=dash2portlets, hasDashboard=found))

        return user2dash



from zope.component import queryAdapter
from zope.annotation.interfaces import IAnnotations

from Products.CMFCore.utils import getToolByName

from plone.portlets.interfaces import ILocalPortletAssignable
from plone.portlets.interfaces import IPortletContext
from plone.portlets.constants import CONTEXT_ASSIGNMENT_KEY
from plone.portlets.constants import CONTEXT_BLACKLIST_STATUS_KEY
from plone.portlets.constants import CONTEXT_CATEGORY
from plone.portlets.retriever import PortletRetriever as basePortletRetriever


class VSPortletRetriever(basePortletRetriever):
    """The vs.dashboardmanager retriever.
    It does the same job as the standard retriever.
    """

    def getPortlets(self):
        """Work out which portlets to display, returning a list of dicts
        describing assignments to render.
        """

        if IPortletContext.providedBy(self.context):
            pcontext = self.context
        else:
            pcontext = queryAdapter(self.context, IPortletContext)

        if pcontext is None:
            return []
            
        # Holds a list of (category, key, assignment).
        categories = [] 
        
        # Keeps track of the blacklisting status for global categores 
        # (user, group, content type). The status is either True (blocked)
        # or False (not blocked).
        blacklisted = {}
                
        # This is the name of the manager (column) we're rendering
        manager = self.storage.__name__
        
        # 1. Fetch blacklisting status for each global category

        # First, find out which categories we will need to determine
        # blacklist status for
        
        for category, key in pcontext.globalPortletCategories(False):
            blacklisted[category] = None
        
        # Then walk the content hierarchy to find out what blacklist status
        # was assigned. Note that the blacklist is tri-state; if it's None it
        # means no assertion has been made (i.e. the category has neither been
        # whitelisted or blacklisted by this object or any parent). The first
        # item to give either a blacklisted (True) or whitelisted (False)
        # value for a given item will set the appropriate value. Parents of
        # this item that also set a black- or white-list value will then be
        # ignored.

        # Whilst walking the hierarchy, we also collect parent portlets, 
        # until we hit the first block.

        current = self.context
        currentpc = pcontext
        blacklistFetched = set()
        parentsBlocked = False
        
        while current is not None and currentpc is not None:
            if ILocalPortletAssignable.providedBy(current):
                assignable = current
            else:
                assignable = queryAdapter(current, ILocalPortletAssignable)

            if assignable is not None:
                if IAnnotations.providedBy(assignable):
                    annotations = assignable
                else:
                    annotations = queryAdapter(assignable, IAnnotations)

                if not parentsBlocked:
                    local = annotations.get(CONTEXT_ASSIGNMENT_KEY, None)
                    if local is not None:
                        localManager = local.get(manager, None)
                        if localManager is not None:
                            categories.extend([(CONTEXT_CATEGORY, currentpc.uid, a) for a in localManager.values()])

                blacklistStatus = annotations.get(CONTEXT_BLACKLIST_STATUS_KEY, {}).get(manager, None)
                if blacklistStatus is not None:
                    for cat, status in blacklistStatus.items():
                        if cat == CONTEXT_CATEGORY:
                            if not parentsBlocked and status == True:
                                parentsBlocked = True
                        else: # global portlet categories
                            if blacklisted.get(cat, False) is None:
                                blacklisted[cat] = status
                            if status is not None:
                                blacklistFetched.add(cat)
                                
            # We can abort if parents are blocked and we've fetched all
            # blacklist statuses
            
            if parentsBlocked and len(blacklistFetched) == len(blacklisted):
                break
                    
            # Check the parent - if there is no parent, we will stop
            current = currentpc.getParent()
            if current is not None:
                if IPortletContext.providedBy(current):
                    currentpc = current
                else:
                    currentpc = queryAdapter(current, IPortletContext)
        
        # Get all global mappings for non-blacklisted categories
        
        for category, key in pcontext.globalPortletCategories(False):
            if not blacklisted[category]:
                mapping = self.storage.get(category, None)
                if mapping is not None:
                    for a in mapping.get(key, {}).values():
                        categories.append((category, key, a,))
        
        assignments = []
        for category, key, assignment in categories:
            assignments.append({'category'    : category,
                                'key'         : key,
                                'name'        : assignment.__name__,
                                'assignment'  : assignment
                                })
        return assignments


################################################################
# vs.dashboardmanager
# (C) 2010, Veit Schiele & Andreas Jung
# Published under the GNU Public Licence V 2 (GPL 2)
################################################################

from plone.app.portlets.interfaces import IDashboard
from plone.portlets.interfaces import IPortletManager
from zope.interface import Interface

class IPortletPageColumn(IDashboard):
    """Marker interface describing columns on a portlet page.
    """
class IDashboardManager(Interface):
    """Marker interface describing dashboardmanager object.
    """


from Testing import ZopeTestCase as ztc

from Products.PloneTestCase import ptc
from Products.PloneTestCase import layer
from Products.Five import zcml
from Products.Five import fiveconfigure

ptc.setupPloneSite(
    extension_profiles=('vs.dashboardmanager:default', )
)

class DashboardManagerLayer(layer.PloneSite):
    """Configure vs.dashboardmanager"""

    @classmethod
    def setUp(cls):
        fiveconfigure.debug_mode = True
        import vs.dashboardmanager
        zcml.load_config("configure.zcml", vs.dashboardmanager)
        fiveconfigure.debug_mode = False
        ztc.installPackage("vs.dashboardmanager", quiet=1)

    @classmethod
    def tearDown(cls):
        pass




################################################################
# vs.dashboardmanager
# (C) 2010, Veit Schiele & Andreas Jung
# Published under the GNU Public Licence V 2 (GPL 2)
################################################################


DEPENDENCIES = (
    'collective.portletpage',
)

PROJECTNAME = 'vs.dashboardmanager'

ADD_PERMISSIONS = {
    'DashboardManager': 'vs.dashboardmanager: Add DashboardManager',
}


import unittest

from Products.CMFCore.utils import getToolByName
from Products.PloneTestCase.ptc import PloneTestCase

from vs.dashboardmanager.tests.layer import DashboardManagerLayer

class UninstallTest(PloneTestCase):

    layer = DashboardManagerLayer

    def uninstall(self):
        setup = getToolByName(self.portal, 'portal_setup')
        setup.runAllImportStepsFromProfile('profile-vs.dashboardmanager:uninstall')

    def test_configlet_uninstall(self):
        self.uninstall()
        controlpanel = getattr(self.portal, 'portal_controlpanel')
        installed = [a.getAction(self)['id'] for a in controlpanel.listActions()]
        self.failIf('vs_dashboard_management' in installed)

def test_suite():
    return unittest.defaultTestLoader.loadTestsFromName(__name__)




import unittest

from zope.component import getMultiAdapter

from Products.CMFCore.utils import getToolByName
from Products.PloneTestCase.ptc import PloneTestCase

from vs.dashboardmanager.tests.layer import DashboardManagerLayer

class DashboardManagerTest(PloneTestCase):

    layer = DashboardManagerLayer

    def test_dashboardmanager_controlpanel_view(self):
        # dashboard manager setting control panel view
        view = getMultiAdapter((self.portal, self.portal.REQUEST), name='dashboard-manager')
        view = view.__of__(self.portal)
        self.failUnless(view())

    def test_dashboardmanager_controlpanel_view_protected(self):
        # dashboard manager setting control panel view can not be accessed by anonymous users
        from AccessControl import Unauthorized
        self.logout()
        self.assertRaises(Unauthorized, self.portal.restrictedTraverse, '@@dashboard-manager')
        
    def test_configlet_install(self):
        controlpanel = getToolByName(self.portal, 'portal_controlpanel')
        installed = [a.getAction(self)['id'] for a in controlpanel.listActions()]
        self.failUnless('vs_dashboard_management' in installed)

def test_suite():
    return unittest.defaultTestLoader.loadTestsFromName(__name__)

